class_name Controller
extends Node

var actions := {} # parsed from steam input manifest

var handle := 0
var type := ""

var state := {}
var previous_state := {}

var vibrate_counter := 0


onready var steam := Steam.Steam as Steam


func start(device: int) -> Node:
	handle = device
	_setup_actions()
	_setup_data()
	return self


func _setup_data() -> void:
	if !is_inside_tree():
		yield(self, "ready")
	type = steam.getInputTypeForHandle(handle)


func _setup_actions() -> void:
	if !is_inside_tree():
		yield(self, "ready")

	actions = VdfParser.parse_file("res://steam_input_manifest.vdf") \
			.get("Action Manifest", {}).get("actions", {})

	for action_set in actions:
		actions[action_set]["id"] = steam.getActionSetHandle(action_set)
		for action in actions[action_set].get("StickPadGyro", {}):
			actions[action_set]["StickPadGyro"][action]["id"] = \
					steam.getAnalogActionHandle(action)
			add_user_signal("%s_changed" % action, [{"name": "value", "type": TYPE_VECTOR2}])
		for action in actions[action_set].get("AnalogTrigger", {}):
			actions[action_set]["AnalogTrigger"][action] = \
					steam.getAnalogActionHandle(action)
			add_user_signal("%s_changed" % action, [{"name": "value", "type": TYPE_REAL}])
		for action in actions[action_set].get("Button", {}):
			actions[action_set]["Button"][action] = \
					steam.getDigitalActionHandle(action)
			add_user_signal("%s_changed" % action, [{"name": "value", "type": TYPE_BOOL}])


func vibrate(weak: float, strong: float, time: float) -> void:
	if !is_inside_tree():
		yield(self, "ready")
	steam.triggerVibration(handle, int(round(strong * 65535)) % 65536, int(round(weak * 65535)) % 65536)
	vibrate_counter += 1
	yield(get_tree().create_timer(time), "timeout")
	vibrate_counter -= 1
	if vibrate_counter == 0:
		steam.triggerVibration(handle, 0, 0)


func mute():
	steam.triggerVibration(handle, 0, 0)


func _process(delta: float) -> void:
	previous_state = state.duplicate(true)
	for action_set in actions:
		steam.activateActionSet(handle, actions[action_set]["id"])
		for action in actions[action_set].get("StickPadGyro", {}):
			var data = steam.getAnalogActionData(handle, \
					actions[action_set]["StickPadGyro"][action]["id"])
			if !data["active"]:
				continue

			# see https://partner.steamgames.com/doc/api/ISteamInput#EControllerSourceMode
			if !(data["mode"] in [5, 7]):
				# relative mode (joystick mouse)
				state[action] = Vector2(0, 0)

			state[action] = state.get(action, Vector2(0, 0)) + Vector2(data["x"], data["y"])

		for action in actions[action_set].get("AnalogTrigger", {}):
			var data = steam.getAnalogActionData(handle, \
					actions[action_set]["AnalogTrigger"][action])
			if !data["active"]:
				continue

			# see https://partner.steamgames.com/doc/api/ISteamInput#EControllerSourceMode
			if !(data["mode"] in [5, 7]):
				# relative mode (joystick mouse)
				state[action] = 0

			state[action] += data["x"]

		for action in actions[action_set].get("Button", {}):
			var data = steam.getDigitalActionData(handle, \
					actions[action_set]["Button"][action])
			if !data["bActive"]:
				continue

			state[action] = data["bState"]

	for action in state:
		if state[action] != previous_state.get(action, null):
			emit_signal("%s_changed" % action, state[action])
