class_name VdfParser
extends Reference


static func parse_file(path: String) -> Dictionary:
	var file := File.new()
	if file.open(path, File.READ) != OK:
		print("Failed to open %s" % path)
		return {}
	return parse(file)


static func parse(file: File) -> Dictionary:
	var data := {}
	var stack := [data] # Stack of Dicts
	var key := ""
	var line := file.get_line().strip_edges()
	while file.get_position() < file.get_len():
		var comment_start := line.find("//")
		if comment_start != -1:
			line = line.substr(0, comment_start).strip_edges()
		if line.empty():
			line = file.get_line().strip_edges()
			continue

		if line[0] == "{":
			var prev = stack.back()
			stack.push_back({})
			prev[key] = stack.back()
			key = ""

			line.erase(0, 1)
			line = line.strip_edges()
			continue
		elif line[0] == "}":
			stack.pop_back()

			line.erase(0, 1)
			line = line.strip_edges()
			continue
		elif line[0] == "\"":
			var length := line.find("\"", 1)
			if length > 0:
				length -= 1
			var word := line.substr(1, length)

			if key.empty():
				key = word
			else:
				stack.back()[key] = word
				key = ""

			if length == -1:
				line = ""
			else:
				line = line.substr(length + 2).strip_edges()
			continue
		else:
			print("Parsing failed. Line: '%s'" % line)
			line = ""
	return data
